!
! anama --- analyze rate equations in Mathematica
! Christian Buth
! 29 October 2016
!
! Version 1.0.0: 10 May 2017 (Output configurations, electronic and radiative transition energies and rates, photoionization cross
!       sections, rate equations, charge state probabilities, fluorescence matrix, energies, and transition descriptions, path
!       weights and strings, graph of the transitions)
! Version 1.1.0: 13 Jun 2017 (Beautified the output and commented the source code)
!
!
! References
! ----------
!
! Wikipedia contributors, 2017, Tuple, Wikipedia, The Free Encyclopedia, May 12, 2017, 20:12 UTC. Available at:
!     http://en.wikipedia.org/w/index.php?title=Tuple&oldid=780075268. Accessed June 6, 2017
!

      program anama
      use fortrantl
      use physconst
      implicit none

      integer, parameter :: maxconf = 5, &                                    ! Maximum number of nonrelativistic subshells accepted
     &                      leninp = 132                                                             ! Maximum length of input lines

      type tabeleccfg
        logical xscTot, &                                                                  ! Configuration undergoes photoionization
     &          augTot, &                                                                               ! Configuration Auger decays
     &          radTot                                                                            ! Configuration decays radiatively
        integer elno, &
     &          cfg(maxconf)                                                       ! Nonrelativistic atomic electronic configuration
      end type tabeleccfg

      type tabtrans
        integer ini, &                                                                                       ! Initial configuration
     &          fin                                                                                            ! Final configuration
        real(dblprec) energy, &                                                                                  ! Transition energy
     &                width                                                                                            ! Decay width
      end type tabtrans

      type, extends(tabtrans) :: tabaugertrans
      end type tabaugertrans

      type, extends(tabtrans) :: tabradiativetrans
      end type tabradiativetrans

      type tabxsect
        integer ini, &                                                                                       ! Initial configuration
     &          fin                                                                                            ! Final configuration
        real(dblprec) ionthresh                                                                        ! Ionization threshold energy
        real(dblprec), allocatable :: energy(:), &                                                             ! Photon energy in eV
     &                                cross(:)                                                              ! Cross section in cm**2
      end type tabxsect

      enum, bind(c)
        enumerator tnone, txsc, taug, trad, tstay                                                                 ! Transition types
      end enum

      type path
        integer nvert, &                                                                  ! Number of vertices traversed by the path
     &          trty(50), &                                                                                        ! Transition type
     &          path(50)                                                                                            ! Configurations
      end type path

      type setofpaths
        integer npath                                                                                   ! Number of paths in the set
        type(path) paths(1000000)                                                                                 ! The actual paths
      end type setofpaths

      logical firstfound                                                ! First occurrence of a quantity found for output formatting

      character(len=2), parameter :: subshellnames(maxconf) = (/'1s', '2s', '2p', '3s', '3p'/)      ! Nonrelativistic subshell names

                                                                                   ! Tuple names from (Wikipedia contributors, 2017)
      character(len=9), parameter :: ionstage(0:10) = (/'Neutral  ', 'Singly   ', 'Doubly   ', 'Triply   ', &
     &                  'Quadruply', 'Quintuply', 'Sextuply ', 'Septuply ', 'Octuply  ', 'Nonuply  ', 'Decuply  '/)

      character(len=leninp) line                                                                         ! Line of input from a file

      integer, parameter :: usum  = 100                                                         ! IO unit for the CAESR summary file

      integer i, j, k, &
     &        atomicnumber, &                                                                                ! Charge of the nucleus
     &        nsubshells, &                                    ! Number of nonrelativistic subshells of the atom in its ground state
     &        nauger, &                                                                                ! Number of Auger transitions
     &        nradiative, &                                                                        ! Number of radiative transitions
     &        ncross, &                                                                                   ! Number of cross sections
     &        nxsects, &                                          ! Number of energies for which cross sections are to be determined
     &        nconfig, &                                                                       ! Number of electronic configurations
     &        cfgno, &                             ! The ordinal number of a configuration in the CAESR table (read in but not used)
     &        elno, &                                                ! The number of electrons of a configuration in the CAESR table
     &        trnno, &                                ! The ordinal number of a transition in the CAESR table (read in but not used)
     &        ini, &                                               ! The ordinal number of the initial configuration of a transition
     &        fin, &                                                 ! The ordinal number of the final configuration of a transition
     &        cfg(maxconf), &                                           ! Subshell occupations of a configuration in the CAESR table
     &        npath, &                                                                                   ! Number of paths generated
     &        trt(50), &                                              ! Initial transition type work array for path finder algorithm
     &        pth(50)                                                                       ! Initial path for path finder algorithm

      real(dblprec) energy, &                                                                            ! Energy in the CAESR table
     &              width, &                                                                        ! Decay width in the CAESR table
     &              cross                                                                         ! Cross section in the CAESR table

      type(tabeleccfg), allocatable :: configs(:), &                  ! The configurations of the atom in all relevant charge states
     &                                 tmpcfg(:)

      type(tabaugertrans), allocatable :: augers(:), &             ! The Auger transitions of the atom in all relevant charge states
     &                                    tmpaug(:)

      type(tabradiativetrans), allocatable :: radiatives(:), & ! The radiative transitions of the atom in all relevant charge states
     &                                        tmprad(:)

      type(tabxsect), allocatable :: crosses(:), &                    ! The cross sections of the atom in all relevant charge states
     &                               tmpcross(:)

      type(setofpaths), allocatable :: pathset(:)                                 ! All possible paths determined by the transitions

      ! Initialization
      write(*,'(A)') '(*'
      write(*,'(A)') '** ANAMA V1.1.0, Analyze rate equations in Mathematica'
      write(*,'(A)') '** (c) 2017 Christian Buth'
      write(*,'(A//)') '*)'

      nconfig = 0
      allocate(configs(1))
      nauger = 0
      allocate(augers(1))
      nradiative = 0
      allocate(radiatives(1))
      ncross = 0
      allocate(crosses(1))

      open(unit=usum, file='neon/caesr_fac.sum', status='old', action='read', form='formatted', access='sequential')
      read(usum, '(A)', end=10) line
      if (trim(line) /= ' CAESR summary file') then
        error stop 'Not a CAESR summary file'
      end if

      ! Read the CAESR sum file into memory
      do
        read(usum, '(A)', end=10) line
        if (line(1:14) == 'Atomic number:') then
          read(line(15:leninp),*) atomicnumber

        else if (line(1:45) == 'Number of nonrelativistic electron subshells:') then
          read(line(46:leninp),*) nsubshells

        else if (line(1:38) == 'Number of energies for cross sections:') then
          read(line(39:leninp),*) nxsects

        else if (line(1:24) == 'Electron configurations:') then
          read(usum, '(5/)')
          do
            read(usum, '(A)') line
            if (line(1:56) == ' -------------------------------------------------------') then
              exit
            end if
            read(line,*) cfgno, elno, (cfg(i), i=1, nsubshells)
            nconfig = nconfig + 1
            if (nconfig > 1) then
              call move_alloc(configs, tmpcfg)
              allocate(configs(nconfig))
              configs(1:nconfig-1) = tmpcfg(:)
              deallocate(tmpcfg)
            end if
            configs(nconfig)%xscTot = .false.
            configs(nconfig)%augTot = .false.
            configs(nconfig)%radTot = .false.
            configs(nconfig)%elno   = elno
            configs(nconfig)%cfg    = cfg
          end do

        else if (line(1:18) == 'Auger transitions:') then
          read(usum, '(5/)')
          do
            read(usum, '(A)') line
            if (line(1:110) == ' -----------------------------------------------------------------' &
     &                      // '--------------------------------------------') then
              exit
            end if
            read(line,*) trnno, ini, fin, energy, width
            energy = energy * eVToHartree
            width = width / 1000.0D0 * eVToHartree
            nauger = nauger + 1
            if (nauger > 1) then
              call move_alloc(augers, tmpaug)
              allocate(augers(nauger))
              augers(1:nauger-1) = tmpaug(:)
              deallocate(tmpaug)
            end if
            augers(nauger)%ini    = ini
            augers(nauger)%fin    = fin
            augers(nauger)%energy = energy
            augers(nauger)%width  = width
          end do

        else if (line(1:22) == 'Radiative transitions:') then
          read(usum, '(5/)')
          do
            read(usum, '(A)') line
            if (line(1:123) == ' -----------------------------------------------------------------' &
     &                      // '---------------------------------------------------------') then
              exit
            end if
            read(line,*) trnno, ini, fin, energy, width
            energy = energy * eVToHartree
            width = width / 1000.0D0 * eVToHartree
            nradiative = nradiative + 1
            if (nradiative > 1) then
              call move_alloc(radiatives, tmprad)
              allocate(radiatives(nradiative))
              radiatives(1:nradiative-1) = tmprad(:)
              deallocate(tmprad)
            end if
            radiatives(nradiative)%ini    = ini
            radiatives(nradiative)%fin    = fin
            radiatives(nradiative)%energy = energy
            radiatives(nradiative)%width  = width
          end do

        else if (line(1:31) == 'Photoionization cross sections:') then
          read(usum, '(4/)')
          do
            read(usum,*) trnno, ini, fin, energy
            energy = energy * eVToHartree
            ncross = ncross + 1
            if (ncross > 1) then
              call move_alloc(crosses, tmpcross)
              allocate(crosses(ncross))
              crosses(1:ncross-1) = tmpcross(:)
              deallocate(tmpcross)
            end if
            crosses(ncross)%ini       = ini
            crosses(ncross)%fin       = fin
            crosses(ncross)%ionthresh = energy
            allocate(crosses(ncross)%energy(nxsects), crosses(ncross)%cross(nxsects))
            read(usum, '(/)')
            do i=1, nxsects
              read(usum,*) energy, cross
              energy = energy * eVToHartree
              cross = cross * cross_section_cm2toAU
              crosses(ncross)%energy(i) = energy
              crosses(ncross)%cross(i)  = cross
            end do
            read(usum, '(3/)', end=10)
          end do
        end if
      end do
 10   close(usum)

      ! Output configuration list
      write(*, '(3(A/))') '(*', '** Configuration list', '*)'
      write(*, '(A)') 'configurations:={'
      do i=1, nconfig
        write(*, '(A,*(I0))', advance='no') '"', configs(i)%cfg(1:nsubshells)
        write(*, '(A)', advance='no') '"'
        if (i /= nconfig) then
          write(*, '(A)', advance='no') ','
          if (mod(i,10) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(/A//)') '}'

      ! Output Auger transition energies
      write(*, '(3(A/))') '(*', '** Auger transition energies [Hartree]', '*)'
      do i=1, nauger
        write(*, '(A,*(I0))', advance='no') 'enA', configs(augers(i)%fin)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no')   'v', configs(augers(i)%ini)%cfg(1:nsubshells)
        write(line, '(F15.6)') augers(i)%energy
        write(*, '(2A)') ':=', trim(adjustl(line(1:15)))
      end do
      write(*, '(/)')

      ! Output partial Auger decay widths
      write(*, '(3(A/))') '(*', '** Partial Auger decay widths [Hartree]', '*)'
      do i=1, nauger
        write(*, '(A,*(I0))', advance='no') 'GamA', configs(augers(i)%fin)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no')    'v', configs(augers(i)%ini)%cfg(1:nsubshells)
        write(line, '(E12.6)') augers(i)%width
        write(*, '(4A)') ':=', line(1:8), '*^', line(10:12)
      end do
      write(*, '(/)')

      ! Output total Auger decay widths
      write(*, '(3(A/))') '(*', '** Total Auger decay widths [Hartree]', '*)'
      do i=1, nconfig
        firstfound = .true.
        do j=1, nauger
          if (augers(j)%ini == i) then
            if (firstfound) then
              write(*, '(A,*(I0))', advance='no') 'GamATot', configs(augers(j)%ini)%cfg(1:nsubshells)
              write(*, '(A)', advance='no') ':='
              configs(augers(j)%ini)%augTot = .true.
              firstfound = .false.
            else
              write(*, '(A)', advance='no') '+'
            end if
            write(*, '(A,*(I0))', advance='no') 'GamA', configs(augers(j)%fin)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no')    'v', configs(augers(j)%ini)%cfg(1:nsubshells)
          end if
        end do
        if (.not. firstfound) then
          write(*, '(A)')
        end if
      end do
      write(*, '(/)')

      ! Output total Auger width list, i.e, True if for the configuration from the list there is a nonzero total Auger decay width;
      ! otherwise False
      write(*, '(3(A/))') '(*', '** List of the existence of a total Auger decay width for a configuration', '*)'
      write(*, '(A/)', advance='no') 'existsGamAtot:={'
      do i=1, nconfig
        if (configs(i)%augTot) then
          write(*, '(A)', advance='no') 'True'
        else
          write(*, '(A)', advance='no') 'False'
        end if
        if (i /= nconfig) then
          write(*, '(A)', advance='no') ','
          if (mod(i,10) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(/A//)') '}'

      ! Output radiative transition energies
      write(*, '(3(A/))') '(*', '** Radiative transition energies [Hartree]', '*)'
      do i=1, nradiative
        write(*, '(A,*(I0))', advance='no') 'enR', configs(radiatives(i)%fin)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no')   'v', configs(radiatives(i)%ini)%cfg(1:nsubshells)
        write(line, '(F15.6)') radiatives(i)%energy
        write(*, '(2A)') ':=', trim(adjustl(line(1:15)))
      end do
      write(*, '(/)')

      ! Output partial radiative widths
      write(*, '(3(A/))') '(*', '** Partial radiative decay widths [Hartree]', '*)'
      do i=1, nradiative
        write(*, '(A,*(I0))', advance='no') 'GamR', configs(radiatives(i)%fin)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no')    'v', configs(radiatives(i)%ini)%cfg(1:nsubshells)
        write(line, '(E12.6)') radiatives(i)%width
        write(*, '(4A)') ':=', line(1:8), '*^', line(10:12)
      end do
      write(*, '(/)')

      ! Output total radiative widths
      write(*, '(3(A/))') '(*', '** Total radiative decay widths [Hartree]', '*)'
      do i=1, nconfig
        firstfound = .true.
        do j=1, nradiative
          if (radiatives(j)%ini == i) then
            if (firstfound) then
              write(*, '(A,*(I0))', advance='no') 'GamRTot', configs(radiatives(j)%ini)%cfg(1:nsubshells)
              write(*, '(A)', advance='no') ':='
              configs(radiatives(j)%ini)%radTot = .true.
              firstfound = .false.
            else
              write(*, '(A)', advance='no') '+'
            end if
            write(*, '(A,*(I0))', advance='no') 'GamR', configs(radiatives(j)%fin)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no')    'v', configs(radiatives(j)%ini)%cfg(1:nsubshells)
          end if
        end do
        if (.not. firstfound) then
          write(*, '(A)')
        end if
      end do
      write(*, '(/)')

      ! Output total radiative width list
      write(*, '(3(A/))') '(*', '** List of the existence of a total radiative decay width for a configuration', '*)'
      write(*, '(A/)', advance='no') 'existsGamRtot:={'
      do i=1, nconfig
        if (configs(i)%radTot) then
          write(*, '(A)', advance='no') 'True'
        else
          write(*, '(A)', advance='no') 'False'
        end if
        if (i /= nconfig) then
          write(*, '(A)', advance='no') ','
          if (mod(i,10) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(/A//)') '}'

      ! Output partial cross sections
      write(*, '(3(A/))') '(*', '** Partial photoionization cross sections [atomic unit]', '*)'
      do i=1, ncross
        write(*, '(A,*(I0))', advance='no') 'xsc', configs(crosses(i)%fin)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no')   'v', configs(crosses(i)%ini)%cfg(1:nsubshells)
        write(*, '(A)') '[en_]:=Module[{fxs}, fxs=Interpolation[{'
        do j=1, nxsects
          write(line, '(F15.6)') crosses(i)%energy(j)
          write(*, '(2A)', advance='no') '{', trim(adjustl(line(1:15)))
          write(line, '(E12.6)') crosses(i)%cross(j)
          write(*, '(5A)', advance='no') ',', line(1:8), '*^', line(10:12), '}'
          if (j == nxsects) then
            write(*, '(A)')
          else
            write(*, '(A)', advance='no') ','
            if (mod(j,3) == 0) then
              write (*,*)
            end if
          end if
        end do
        write(line, '(F15.6)') crosses(i)%energy(1)
        write(*, '(2A)', advance='no') '},Method->"Spline",InterpolationOrder->1]; fxs[en]*UnitStep[en-', &
     &                                 trim(adjustl(line(1:15)))
        write(line, '(F15.6)') crosses(i)%energy(nxsects)
        write(*, '(3A)') ']*UnitStep[', trim(adjustl(line(1:15))), '-en]]'
        write(*, '(/)')
      end do

      ! Output total cross sections
      write(*, '(3(A/))') '(*', '** Total photoionization cross sections [atomic unit]', '*)'
      do i=1, nconfig
        firstfound = .true.
        do j=1, ncross
          if (crosses(j)%ini == i) then
            if (firstfound) then
              write(*, '(A,*(I0))', advance='no') 'xscTot', configs(crosses(j)%ini)%cfg(1:nsubshells)
              write(*, '(A)', advance='no') '[en_]:='
              configs(crosses(j)%ini)%xscTot = .true.
              firstfound = .false.
            else
              write(*, '(A)', advance='no') '+'
            end if
            write(*, '(A,*(I0))', advance='no') 'xsc', configs(crosses(j)%fin)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no')   'v', configs(crosses(j)%ini)%cfg(1:nsubshells)
            write(*, '(A)', advance='no') '[en]'
          end if
        end do
        if (.not. firstfound) then
          write(*,*)
        end if
      end do
      write(*, '(/)')

      ! Output total cross section list
      write(*, '(3(A/))') '(*', '** List of the existence of a total cross section for a configuration', '*)'
      write(*, '(A/)', advance='no') 'existsxsctot:={'
      do i=1, nconfig
        if (configs(i)%xscTot) then
          write(*, '(A)', advance='no') 'True'
        else
          write(*, '(A)', advance='no') 'False'
        end if
        if (i /= nconfig) then
          write(*, '(A)', advance='no') ','
          if (mod(i,10) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(/A//)') '}'

      ! Output rate equations
      write(*, '(3(A/))') '(*', '** Rate equations', '*)'
      do i=1, nconfig
        write(*, '(A,*(I0))', advance='no') 'req', configs(i)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no') '[en_,jx_]:=P', configs(i)%cfg(1:nsubshells)
        write(*, '(A)', advance='no') '''[t]=='
        firstfound = .true.

        do j=1, ncross
          if (crosses(j)%fin == i) then
            if (firstfound) then
              firstfound = .false.
            else
              write(*, '(A)', advance='no') '+'
            end if
            write(*, '(A,*(I0))', advance='no') 'xsc', configs(crosses(j)%fin)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no')   'v', configs(crosses(j)%ini)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no') '[en]*jx[t]*P', configs(crosses(j)%ini)%cfg(1:nsubshells)
            write(*, '(A)', advance='no') '[t]'
          end if
        end do
        if (configs(i)%xscTot) then
          write(*, '(A,*(I0))', advance='no') '-xscTot', configs(i)%cfg(1:nsubshells)
          write(*, '(A,*(I0))', advance='no') '[en]*jx[t]*P', configs(i)%cfg(1:nsubshells)
          write(*, '(A)', advance='no') '[t]'
        end if

        do j=1, nauger
          if (augers(j)%fin == i) then
            write(*, '(A,*(I0))', advance='no') '+GamA', configs(augers(j)%fin)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no')     'v', configs(augers(j)%ini)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no') '*P', configs(augers(j)%ini)%cfg(1:nsubshells)
            write(*, '(A)', advance='no') '[t]'
          end if
        end do
        if (configs(i)%augTot) then
          write(*, '(A,*(I0))', advance='no') '-GamATot', configs(i)%cfg(1:nsubshells)
          write(*, '(A,*(I0))', advance='no') '*P', configs(i)%cfg(1:nsubshells)
          write(*, '(A)', advance='no') '[t]'
        end if

        do j=1, nradiative
          if (radiatives(j)%fin == i) then
            write(*, '(A,*(I0))', advance='no') '+GamR', configs(radiatives(j)%fin)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no')     'v', configs(radiatives(j)%ini)%cfg(1:nsubshells)
            write(*, '(A,*(I0))', advance='no') '*P', configs(radiatives(j)%ini)%cfg(1:nsubshells)
            write(*, '(A)', advance='no') '[t]'
          end if
        end do
        if (configs(i)%radTot) then
          write(*, '(A,*(I0))', advance='no') '-GamRTot', configs(i)%cfg(1:nsubshells)
          write(*, '(A,*(I0))', advance='no') '*P', configs(i)%cfg(1:nsubshells)
          write(*, '(A)', advance='no') '[t]'
        end if

        write(*, '(2/)')
      end do

      ! Output rate-equation solver
      write(*, '(3(A/))') '(*', '** Rate-equation solver', '*)'
      write(*, '(A)') 'solveRateEquations[en_,jx_,neginf_,posinf_,maxstfr_]:=(NDSolve[{'
      do i=1, nconfig
        write(*, '(A,*(I0))', advance='no') 'req', configs(i)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no') '[en,jx],P', configs(i)%cfg(1:nsubshells)
        write(*, '(A)', advance='no') '[neginf]=='
        if (i == 1) then
          write(*, '(A)', advance='no') '1,'
        else if (i == nconfig) then
          write(*, '(A/A)') '0', '},{'
        else
          write(*, '(A)', advance='no') '0,'
          if (mod(i,3) == 0) then
            write (*,*)
          end if
        end if
      end do
      do i=1, nconfig
        write(*, '(A,*(I0))', advance='no') 'P', configs(i)%cfg(1:nsubshells)
        if (i == nconfig) then
          write(*, '(A)') '[t]'
        else
          write(*, '(A)', advance='no') '[t],'
          if (mod(i,10) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(A//)') '},{t,neginf,posinf},MaxStepFraction->maxstfr])'

      ! Output charge-state probabilities
      write(*, '(3(A/))') '(*', '** Charge-state probabilities', '*)'
      do i=configs(1)%elno, 0, -1
        write(*, '(3A)', advance='no') 'sol', trim(ionstage(10-i)),'[sol_,t1_]:=('
        firstfound = .true.
        do j=1, nconfig
          if (configs(j)%elno == i) then
            if (firstfound) then
              firstfound = .false.
            else
              write(*, '(A)', advance='no') '+'
            end if
            write(*, '(A,*(I0))', advance='no') 'Replace[P', configs(j)%cfg(1:nsubshells)
            write(*, '(A)', advance='no') '[t],sol[[1]]]'
          end if
        end do
        write(*, '(A//)') ')/.{t->t1}'
      end do

      ! Output fluorescence matrix
      write(*, '(3(A/))') '(*', '** Fluorescence matrix [Hartree]', '*)'
      write(*, '(A)') 'fluorecencematrix:=SparseArray[{'
      do i=1, nradiative
        write(*, '(A,2(I0,A))', advance='no') '{', i, ',', radiatives(i)%ini, '}->'
        write(*, '(A,*(I0))', advance='no') 'GamR', configs(radiatives(i)%fin)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no')    'v', configs(radiatives(i)%ini)%cfg(1:nsubshells)
        if (i /= nradiative) then
          write(*, '(A)', advance='no') ','
          if (mod(i,4) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(/A,2(I0,A)//)') '},{', nradiative, ',', nconfig,'}]'

      ! Output fluorescence energies
      write(*, '(3(A/))') '(*', '** Fluorescence energies [Hartree]', '*)'
      write(*, '(A)') 'fluorecenceenergy:={'
      do i=1, nradiative
        write(line, '(F15.6)') radiatives(i)%energy
        write(*, '(A,*(I0))', advance='no') 'enR', configs(radiatives(i)%fin)%cfg(1:nsubshells)
        write(*, '(A,*(I0))', advance='no')   'v', configs(radiatives(i)%ini)%cfg(1:nsubshells)
        if (i /= nradiative) then
          write(*, '(A)', advance='no') ','
          if (mod(i,7) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(/A//)') '}'

      ! Output fluorescence transition list
      write(*, '(3(A/))') '(*', '** Fluorescence transition list', '*)'
      write(*, '(A)') 'fluorescencetransitions:={'
      do i=1, nradiative
        write(*, '(A)', advance='no') '"'
        do j=1, nsubshells
          write(*, '(A,I0,A)', advance='no') subshellnames(j) // '(', configs(radiatives(i)%ini)%cfg(j), ') '
        end do
        write(*, '(A)', advance='no') '\[LongRightArrow] '
        do j=1, nsubshells-1
          write(*, '(A,I0,A)', advance='no') subshellnames(j) // '(', configs(radiatives(i)%fin)%cfg(j), ') '
        end do
        write(*, '(A,I0,A)', advance='no') subshellnames(nsubshells) // '(', configs(radiatives(i)%fin)%cfg(nsubshells), ')"'
        if (i /= nradiative) then
          write(*, '(A)', advance='no') ','
          if (mod(i,2) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(/A//)') '}'

      ! Output graph of transitions
      write(*, '(3(A/))') '(*', '** Graph of transitions', '*)'
      write(*, '(A)') 'edges:={'
      write(*, '(A)') '(* Photoionizations *)'
      do i=1, ncross
        write(*, '(A,2(I0,A))', advance='no') 'Style[', crosses(i)%ini, '->', crosses(i)%fin, ',Red],'
        if (i /= ncross .and. mod(i,5) == 0) then
          write (*,*)
        end if
      end do
      write(*, '(/A)') '(* Auger transitions *)'
      do i=1, nauger
        write(*, '(A,2(I0,A))', advance='no') 'Style[', augers(i)%ini, '->', augers(i)%fin, ',Green],'
        if (i /= nauger .and. mod(i,5) == 0) then
          write (*,*)
        end if
      end do
      write(*, '(/A)') '(* Radiative transitions *)'
      do i=1, nradiative
        write(*, '(A,2(I0,A))', advance='no') 'Style[', radiatives(i)%ini, '->', radiatives(i)%fin, ',Black]'
        if (i /= nradiative) then
          write(*, '(A)', advance='no') ','
          if (mod(i,5) == 0) then
            write (*,*)
          end if
        end if
      end do
      write(*, '(/A)') '}'


!
! Continue only for neon
!

      ! Assure that the remainder is only executed for neon; requires verification for other atoms
      if (atomicnumber /= 10) then
        error stop 'Path analysis is only implemented for neon'
      end if

      ! Generate paths
      write(*, '(//3(A/))') '(*', '** Path finder algorithm', '*)'
      allocate(pathset(nconfig))
      do i=1, nconfig
        pathset(i)%npath = 0
        do j=1, 100000
          pathset(i)%paths(j)%nvert = 0
          pathset(i)%paths(j)%trty  = tnone
          pathset(i)%paths(j)%path  = 0
        end do
      end do
      trt = tnone
      pth(1) = 1
      pth(2:50) = 0
      npath = 0
      call pathfinder(trt,pth,1,1,npath)
      write(*, '(A,I0,A)') '(* Number of paths generated: ', npath, ' *)'
      write(*, '(/A//)') 'safeTot[x_]:=If[x==0,1,x]'

      ! Output paths weights
      write(*, '(3(A/))') '(*', '** Path weights', '*)'
      write(*, '(A)') 'pathset:={'
      do i=1, nconfig
        write(*, '(A)') '{'
        ! Output unity for the neutral neon atom
        if (i == 1) then
          write(*, '(A)', advance='no') '1'
        end if
        ! Output weight of the path
        do j=1, pathset(i)%npath
          do k=pathset(i)%paths(j)%nvert, 2, -1
            ini = pathset(i)%paths(j)%path(k-1)
            fin = pathset(i)%paths(j)%path(k)
            if (pathset(i)%paths(j)%trty(k-1) == txsc) then
              write(*, '(A,*(I0))', advance='no') 'xsc', configs(fin)%cfg(1:nsubshells)
              write(*, '(A,*(I0))', advance='no')   'v', configs(ini)%cfg(1:nsubshells)
              write(*, '(A,*(I0))', advance='no') '[omegaX] / safeTot[xscTot', configs(ini)%cfg(1:nsubshells)
              write(*, '(A,I0,A)', advance='no') '[omegaX]] * branching[[', ini, ',1]]'
            else if (pathset(i)%paths(j)%trty(k-1) == taug) then
              write(*, '(A,*(I0))', advance='no') 'GamA', configs(fin)%cfg(1:nsubshells)
              write(*, '(A,*(I0))', advance='no')    'v', configs(ini)%cfg(1:nsubshells)
              write(*, '(A,*(I0))', advance='no') ' / GamATot', configs(ini)%cfg(1:nsubshells)
              write(*, '(A,I0,A)', advance='no') ' * branching[[', ini, ',2]]'
            else if (pathset(i)%paths(j)%trty(k-1) == trad) then
              write(*, '(A,*(I0))', advance='no') 'GamR', configs(fin)%cfg(1:nsubshells)
              write(*, '(A,*(I0))', advance='no')    'v', configs(ini)%cfg(1:nsubshells)
              write(*, '(A,*(I0))', advance='no') ' / GamRTot', configs(ini)%cfg(1:nsubshells)
              write(*, '(A,I0,A)', advance='no') ' * branching[[', ini, ',3]]'
            else
              error stop 'Unknown transition type'
            end if
            if (k /= 2) then
              write(*, '(A)', advance='no') ' * '
            end if
          end do
          if (j /= pathset(i)%npath) then
            write(*, '(A)') ','
          else
            write(*,*)
          end if
        end do
        write(*, '(A)', advance='no') '}'
        if (i /= nconfig) then
          write(*, '(A)') ','
        end if
      end do
      write(*, '(/A//)') '}'

      ! Output paths strings
      write(*, '(3(A/))') '(*', '** Path strings', '*)'
      write(*, '(A)') 'pathstr:={'
      do i=1, nconfig
        write(*, '(A)') '{'
        ! Output string of the path
        do j=1, pathset(i)%npath
          write(*, '(A)', advance='no') '"'
          do k=2, pathset(i)%paths(j)%nvert
            ini = pathset(i)%paths(j)%path(k-1)
            fin = pathset(i)%paths(j)%path(k)
            if (pathset(i)%paths(j)%trty(k-1) == txsc) then
              if (configs(ini)%cfg(1) == 2 .and. configs(fin)%cfg(1) == 1) then
                write(*, '(A)', advance='no') 'C'
              else if (configs(ini)%cfg(1) == 1 .and. configs(fin)%cfg(1) == 0) then
                write(*, '(A)', advance='no') 'D'
              else
                write(*, '(A)', advance='no') 'V'
              end if
            else if (pathset(i)%paths(j)%trty(k-1) == taug) then
              write(*, '(A)', advance='no') 'A'
            else if (pathset(i)%paths(j)%trty(k-1) == trad) then
              write(*, '(A)', advance='no') 'R'
            else
              error stop 'Unknown transition type'
            end if
          end do
          if (j /= pathset(i)%npath) then
            write(*, '(A)', advance='no') '",'
            if (mod(j,5) == 0) then
              write (*,*)
            end if
          else
            write(*, '(A)') '"'
          end if
        end do
        write(*, '(A)', advance='no') '}'
        if (i /= nconfig) then
          write(*, '(A)') ','
        end if
      end do
      write(*, '(/A)') '}'

      ! Clean up
      deallocate(pathset)
      write (*,'(//A)') '(* end ANAMA Mathematica file *)'


      contains
      
!
! pathfinder --- recursively find paths
!

      recursive subroutine pathfinder(trty, path, ipt, ini, npath)
      implicit none

      integer, intent(in) :: trty(50), path(50), ipt, ini
      integer, intent(inout) :: npath
      integer i, newtr(50), newpt(50)

      npath = npath + 1
      pathset(ini)%npath = pathset(ini)%npath + 1
      pathset(ini)%paths(pathset(ini)%npath)%nvert = ipt
      pathset(ini)%paths(pathset(ini)%npath)%trty = trty
      pathset(ini)%paths(pathset(ini)%npath)%trty(ipt) = tstay
      pathset(ini)%paths(pathset(ini)%npath)%path = path

      do i=1, ncross
        if (crosses(i)%ini == ini) then
          newtr = trty
          newtr(ipt) = txsc
          newpt = path
          newpt(ipt+1) = crosses(i)%fin
          call pathfinder(newtr,newpt,ipt+1,crosses(i)%fin,npath)
        end if
      end do

      do i=1, nauger
        if (augers(i)%ini == ini) then
          newtr = trty
          newtr(ipt) = taug
          newpt = path
          newpt(ipt+1) = augers(i)%fin
          call pathfinder(newtr,newpt,ipt+1,augers(i)%fin,npath)
        end if
      end do

 rad: do i=1, nradiative
        if (radiatives(i)%ini == ini) then
          ! Only include radiative transition if there is no competing Auger transition
          do j=1, nauger
            if (augers(j)%ini == ini) then
              exit rad
            end if
          end do

          newtr = trty
          newtr(ipt) = trad
          newpt = path
          newpt(ipt+1) = radiatives(i)%fin
          call pathfinder(newtr,newpt,ipt+1,radiatives(i)%fin,npath)
        end if
      end do rad
      end subroutine pathfinder
      end program anama