
MODULE FiniteElements
  implicit none

  PRIVATE
  PUBLIC :: CreateNodes, FEbasfun, FEbasd1, PrintNodes, node_spacing, &
          & numbasfun, CreateFEBmat1d

  TYPE, PUBLIC :: FEBmat1d
    INTEGER :: n
    REAL(kind=8), pointer :: M(:,:)
  END TYPE FEBmat1d

  TYPE, PUBLIC :: FEBnodes
    INTEGER :: maxnumnds=0, numnds, numleft, numright, numbasis
    REAL(kind=8), POINTER :: nodes(:)
  END TYPE FEBnodes

  INTERFACE CreateFEBmat1d
    MODULE PROCEDURE CreateFEBmat1d1, CreateFEBmat1d2
  END INTERFACE

  INTERFACE CreateNodes
    MODULE PROCEDURE CreateNodes1, CreateNodes2, CreateNodes3, CreateNodes4
  END INTERFACE

  INTERFACE FEbasfun
    MODULE PROCEDURE FEbasfun0, FEbasfun1
  END INTERFACE

  INTERFACE FEbasd1
    MODULE PROCEDURE FEbasd10, FEbasd11
  END INTERFACE

CONTAINS


SUBROUTINE CreateFEBmat1d2(x,nds)
  TYPE(FEBmat1d) :: x
  TYPE(FEBnodes) :: nds

  x%n = numbasfun(nds)
  IF ( ASSOCIATED(x%M) ) THEN
    DEALLOCATE( x%M )
  ENDIF
  ALLOCATE( x%M(-5:5,x%n) )
  x%M(:,:) = 0.d0

  RETURN
END SUBROUTINE CreateFEBmat1d2


SUBROUTINE CreateFEBmat1d1(x,nbasis)
  TYPE(FEBmat1d) :: x
  INTEGER :: nbasis

  x%n = nbasis
  IF ( ASSOCIATED(x%M) ) THEN
    DEALLOCATE( x%M )
  ENDIF
  IF (nbasis > 0) THEN
    ALLOCATE( x%M(-5:5,x%n) )
    x%M(:,:) = 0.d0
  ENDIF

  RETURN
END SUBROUTINE CreateFEBmat1d1


SUBROUTINE PrintNodes(x,dvc)
  TYPE(FEBnodes) :: x
  INTEGER :: dvc, j

  DO j = 1,x%numnds
    WRITE(dvc,110) j, x%nodes(j)
  END DO
  WRITE(dvc,120) x%numbasis

  110 FORMAT(I4,':  ',F8.3)
  120 FORMAT('total of ', I5, ' basis functions')

  RETURN
END SUBROUTINE PrintNodes


SUBROUTINE CreateNodes1(x,xmin,xmax,numint,leftright)
  TYPE(FEBnodes) :: x
  INTEGER :: numint, j
  INTEGER, OPTIONAL :: leftright(2)
  REAL(kind=8) :: xmin, xmax, dx
  
  CALL initialize_storage(x,numint+1)
  if (x%maxnumnds <= 0) RETURN
  dx = (xmax-xmin)/(numint*1.d0)
  x%nodes(1) = xmin
  DO j = 2, numint
    x%nodes(j) = 0.5d0*( xmin + xmax + (2*j-numint-2)*dx )
  END DO
  x%nodes(numint+1) = xmax
  x%numnds = numint+1

  IF (PRESENT(leftright)) THEN
    x%numleft = MIN(leftright(1),3)
    x%numleft = MAX(x%numleft,0)
    x%numright = MIN(leftright(2),3)
    x%numright = MAX(x%numright,0)
  ELSE
    x%numleft = 2
    x%numright = 2
  ENDIF
  x%numbasis = numbasfun(x)

  RETURN
END SUBROUTINE CreateNodes1


SUBROUTINE CreateNodes2(x,xmin,xmax,dx_in,leftright)
  TYPE(FEBnodes) :: x
  REAL(kind=8) :: xmin, xmax, dx_in, dx, length, diff
  INTEGER, OPTIONAL :: leftright(2)
  INTEGER :: n, j

  dx = abs(dx_in)
  length = abs(xmax - xmin)
  IF ( dx*length <= 1.d-8 ) THEN
    CALL destroy_storage(x)
    RETURN
  ENDIF
  n = length/dx
  IF (n < 1) THEN
    n = 1
    dx = length
  ENDIF
  diff = length - n*dx
  if (diff >= 0.249d0*dx) n = n + 1
  CALL initialize_storage(x,n+1)
  if (xmax < xmin) dx = -dx
  x%nodes(1) = xmin
  DO j = 1,n
    x%nodes(j+1) = xmin + j*dx
  END DO
  diff = xmax - x%nodes(n+1)
  IF (xmax == 0.d0) THEN
    x%nodes(:) = x%nodes(:) + diff
    x%nodes(n+1) = 0.d0
  ELSEIF ( abs(abs(xmax)-abs(xmin)) <= 0.5d0*max(abs(xmax),abs(xmin)) ) THEN
    x%nodes(:) = x%nodes(:) + 0.5d0*diff
  ENDIF
  x%numnds = n+1

  IF (PRESENT(leftright)) THEN
    x%numleft = MIN(leftright(1),3)
    x%numleft = MAX(x%numleft,0)
    x%numright = MIN(leftright(2),3)
    x%numright = MAX(x%numright,0)
  ELSE
    x%numleft = 2
    x%numright = 2
  ENDIF
  x%numbasis = numbasfun(x)

  RETURN
END SUBROUTINE CreateNodes2


SUBROUTINE CreateNodes3(x,fname,leftright)
  TYPE(FEBnodes) :: x
  CHARACTER(len=*), intent(IN) :: fname
  INTEGER, OPTIONAL :: leftright(2)
  INTEGER :: n, j, ios
  REAL(kind=8), ALLOCATABLE :: tmp(:)
  INTEGER, PARAMETER :: buffsize=1000

  OPEN (UNIT=7, FILE=fname, STATUS="OLD", ACTION='READ', IOSTAT=ios)
  IF (ios/=0) THEN
    WRITE(*,*) 'ERROR: file ', fname, ' does not exist'
    CALL destroy_storage(x)
    RETURN
  ENDIF
  n = 0
  ALLOCATE( tmp(buffsize) )
  DO j = 1,buffsize
    READ(UNIT=7, FMT=*, IOSTAT=ios) tmp(j)
    IF (ios<0) THEN
      EXIT
    ELSE
      n = n + 1
    ENDIF
  END DO
  CLOSE(7)

  CALL initialize_storage(x,n)
  x%nodes(1:n) = tmp(1:n)
  DEALLOCATE( tmp )
  x%numnds = n

  IF (PRESENT(leftright)) THEN
    x%numleft = MIN(leftright(1),3)
    x%numleft = MAX(x%numleft,0)
    x%numright = MIN(leftright(2),3)
    x%numright = MAX(x%numright,0)
  ELSE
    x%numleft = 2
    x%numright = 2
  ENDIF
  x%numbasis = numbasfun(x)

  RETURN
END SUBROUTINE CreateNodes3

SUBROUTINE CreateNodes4(x,nodes,num,leftright)
  TYPE(FEBnodes) :: x
  INTEGER :: num
  REAL(kind=8) :: nodes(num)
  INTEGER, OPTIONAL :: leftright(2)
  
  WRITE(*,*) 'inside CreateNodes4'
  WRITE(*,*) 'num = ', num
  CALL initialize_storage(x,num)
  if (x%maxnumnds <= 0) RETURN
  WRITE(*,*) 'copying the nodes'
  x%nodes(:) = nodes(:)
  x%numnds = num

  WRITE(*,*) 'setting up left/right'
  IF (PRESENT(leftright)) THEN
    x%numleft = MIN(leftright(1),3)
    x%numleft = MAX(x%numleft,0)
    x%numright = MIN(leftright(2),3)
    x%numright = MAX(x%numright,0)
  ELSE
    x%numleft = 2
    x%numright = 2
  ENDIF
  WRITE(*,*) 'calculating numbasis'
  x%numbasis = numbasfun(x)

  RETURN
END SUBROUTINE CreateNodes4



! FEbasfun evaluates any 3 of the left half-basis
! functions at the grid points x. The half-basis
! functions are assumed defined between 0 and 1.

FUNCTION FEbasfun0(x,mu)
  INTEGER :: mu
  REAL(kind=8) :: x, FEbasfun0

  SELECT CASE (mu)
  CASE (0)
    FEbasfun0 = (x**3)*(10.d0 - x*(15.d0 - 6.d0*x))
  CASE (1)
    FEbasfun0 = -(x**3)*(4.d0 - x*(7.d0 - 3.d0*x))
  CASE (2)
    FEbasfun0 = 0.5d0*(x**3)*(1.d0 - x*(2.d0 - x))
  CASE DEFAULT
    FEbasfun0 = 0.d0
  END SELECT
  RETURN
END FUNCTION FEbasfun0


FUNCTION FEbasfun1(x,mu)
  INTEGER :: mu
  REAL(kind=8) :: x(:), FEbasfun1(SIZE(x))

  SELECT CASE (mu)
  CASE (0)
    FEbasfun1(:) = (x(:)**3)*(10.d0 - x(:)*(15.d0 - 6.d0*x(:)))
  CASE (1)
    FEbasfun1(:) = -(x(:)**3)*(4.d0 - x(:)*(7.d0 - 3.d0*x(:)))
  CASE (2)
    FEbasfun1(:) = 0.5d0*(x(:)**3)*(1.d0 - x(:)*(2.d0 - x(:)))
  CASE DEFAULT
    FEbasfun1(:) = 0.d0
  END SELECT
  RETURN
END FUNCTION FEbasfun1


! FEbasd1 evaluates any 3 of the first derivatives of the left
! half-basis functions at the grid points x. The half-basis
! functions are assumed defined between 0 and 1.

FUNCTION FEbasd10(x,mu)
  INTEGER :: mu
  REAL(kind=8) :: x, FEbasd10

  SELECT CASE (mu)
  CASE (0)
    FEbasd10 = 30.d0*x*x*(1.d0 - x*(2.d0 - 1.d0*x))
  CASE (1)
    FEbasd10 = -x*x*(12.d0 - x*(28.d0 - 15.d0*x))
  CASE (2)
    FEbasd10 = 0.5d0*x*x*(3.d0 - x*(8.d0 - 5.d0*x))
  CASE DEFAULT
    FEbasd10 = 0.d0
  END SELECT
  RETURN
END FUNCTION FEbasd10


FUNCTION FEbasd11(x,mu)
  INTEGER :: mu
  REAL(kind=8) :: x(:), FEbasd11(SIZE(x))

  SELECT CASE (mu)
  CASE (0)
    FEbasd11(:) = 30.d0*x*x*(1.d0 - x*(2.d0 - 1.d0*x))
  CASE (1)
    FEbasd11(:) = -x*x*(12.d0 - x*(28.d0 - 15.d0*x))
  CASE (2)
    FEbasd11(:) = 0.5d0*x*x*(3.d0 - x*(8.d0 - 5.d0*x))
  CASE DEFAULT
    FEbasd11(:) = 0.d0
  END SELECT
  RETURN
END FUNCTION FEbasd11


FUNCTION numbasfun(x)
  TYPE(FEBnodes) :: x
  INTEGER :: numbasfun

  numbasfun = 3*(x%numnds-2) + x%numleft + x%numright
  RETURN
END FUNCTION numbasfun


FUNCTION node_spacing(x,j)
  INTEGER :: j
  TYPE(FEBnodes) :: x
  REAL(kind=8) :: node_spacing

  IF ( (j<1) .OR. (j>=x%numnds) ) THEN
    node_spacing = 0.d0
  ELSE
    node_spacing = x%nodes(j+1) - x%nodes(j)
  ENDIF
  RETURN
END FUNCTION node_spacing


SUBROUTINE initialize_storage(x,n)
  TYPE(FEBnodes) :: x
  INTEGER :: n
  INTEGER :: stat

  IF ( ASSOCIATED(x%nodes) ) THEN
    IF ( (x%maxnumnds < n) .OR. (x%maxnumnds > 10*n) ) THEN
      DEALLOCATE( x%nodes, STAT=stat )
      if (stat/=0) WRITE(*,*) 'deallocation error'
      ALLOCATE( x%nodes(n), STAT=stat )
      if (stat/=0) WRITE(*,*) 'reallocation error'
    ENDIF
  ELSE
    IF ( n > 0 ) THEN
      ALLOCATE( x%nodes(n), STAT=stat )
      if (stat/=0) WRITE(*,*) 'allocation error'
      x%maxnumnds = n
    ELSE
      x%maxnumnds = 0
    ENDIF
  ENDIF
  IF (x%maxnumnds > 0) THEN
    x%nodes(:) = 0.d0
  ENDIF
  x%numnds = 0
  x%numleft = 2
  x%numright = 2
  x%numbasis = 0
  RETURN
END SUBROUTINE initialize_storage


SUBROUTINE destroy_storage(x)
  TYPE(FEBnodes) :: x
  INTEGER :: ierr

  DEALLOCATE( x%nodes, STAT=ierr )
  x%maxnumnds = 0
  x%numnds = 0
  x%numleft = 2
  x%numright = 2
  x%numbasis = 0
  RETURN
END SUBROUTINE destroy_storage

END MODULE FiniteElements
